/* 
 *  EvRA - Event Routing Application
 *
 *  Copyright (C) 2004  Simon Budig <simon@budig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib-object.h>

#include "evraaction.h"


enum
{
  FEEDBACK_CHANGED,
  LAST_SIGNAL
};


static void    evra_action_class_init        (EvraActionClass  *klass);
static void    evra_action_init              (EvraAction       *object);
static gdouble evra_action_real_get_feedback (const EvraAction *action);


static guint   action_signals[LAST_SIGNAL] = { 0 };

static GObjectClass *parent_class = NULL;


GType
evra_action_get_type (void)
{
  static GType object_type = 0;

  if (! object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (EvraActionClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) evra_action_class_init,
	NULL,           /* class_finalize */
	NULL,           /* class_data     */
	sizeof (EvraAction),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) evra_action_init,
      };

      object_type = g_type_register_static (G_TYPE_OBJECT,
					    "EvraAction",
					    &object_info, 0);
    }

  return object_type;
}

static void
evra_action_class_init (EvraActionClass *klass)
{
  GObjectClass *action_class;

  action_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  action_signals[FEEDBACK_CHANGED] =
    g_signal_new ("feedback_changed",
		  G_TYPE_FROM_CLASS (klass),
		  G_SIGNAL_RUN_FIRST,
		  G_STRUCT_OFFSET (EvraActionClass, feedback_changed),
		  NULL, NULL,
		  g_cclosure_marshal_VOID__VOID,
		  G_TYPE_NONE, 0);

  klass->handle_event        = NULL;
  klass->get_feedback        = evra_action_real_get_feedback;
}


static void
evra_action_init (EvraAction *object)
{
  object->feedback = 0.0;
}


void
evra_action_handle_event (EvraAction  *action,
		          EvraEvent   *event)
{
  EvraActionClass *action_class;

  g_return_if_fail (EVRA_IS_ACTION (action));

  action_class = EVRA_ACTION_GET_CLASS (action);

  if (action_class->handle_event)
    action_class->handle_event (action, event);
}


void
evra_action_feedback_changed (EvraAction *action)
{
  g_return_if_fail (EVRA_IS_ACTION (action));

  g_signal_emit (action, action_signals[FEEDBACK_CHANGED], 0);
}


gdouble
evra_action_get_feedback (const EvraAction *action)
{
  EvraActionClass *action_class;

  g_return_val_if_fail (EVRA_IS_ACTION (action), -1.0);
  action_class = EVRA_ACTION_GET_CLASS (action);
  g_return_val_if_fail (action_class->get_feedback != NULL, -1.0);

  return action_class->get_feedback (action);
}


static gdouble
evra_action_real_get_feedback (const EvraAction *action)
{
  g_return_val_if_fail (EVRA_IS_ACTION (action), -1.0);

  return action->feedback;
}

