/* 
 *  EvRA - Event Routing Application
 *
 *  Copyright (C) 2004  Simon Budig <simon@budig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <glib-object.h>

#include "evraxtestaction.h"

#include <X11/extensions/XTest.h>


enum
{
  FEEDBACK_CHANGED,
  LAST_SIGNAL
};


static void    evra_xtest_action_class_init   (EvraXtestActionClass *klass);
static void    evra_xtest_action_handle_event (EvraAction           *action,
                                               EvraEvent            *event);
static void    evra_xtest_action_send_key     (EvraXtestAction      *action,
                                               gchar                *keysymname);


static EvraActionClass *parent_class = NULL;

static Display *xdisplay = NULL;


GType
evra_xtest_action_get_type (void)
{
  static GType object_type = 0;

  if (! object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (EvraActionClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) evra_xtest_action_class_init,
	NULL,           /* class_finalize */
	NULL,           /* class_data     */
	sizeof (EvraXtestAction),
	0,              /* n_preallocs    */
	NULL,           /* instance_init  */
      };

      object_type = g_type_register_static (EVRA_TYPE_ACTION,
					    "EvraXtestAction",
					    &object_info, 0);
    }

  return object_type;
}

static void
evra_xtest_action_class_init (EvraXtestActionClass *klass)
{
  GObjectClass *action_class;

  action_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

  parent_class->handle_event = evra_xtest_action_handle_event;

  if ((xdisplay = XOpenDisplay (XDisplayName (NULL))) == NULL)
    {
      g_printerr ("Can't connect to X display - "
                  "No X11-Events will be synthesized\n");
    }

}


static void
evra_xtest_action_handle_event (EvraAction  *action,
		                EvraEvent   *event)
{
  EvraXtestAction *xaction;

  g_return_if_fail (EVRA_IS_XTEST_ACTION (action));

  xaction = EVRA_XTEST_ACTION (action);

  switch (*event)
    {
      case EVRA_TURN_LEFT:
          evra_xtest_action_send_key (xaction, "Left");
          break;
      case EVRA_TURN_RIGHT:
          evra_xtest_action_send_key (xaction, "Right");
          break;
      default:
          break;
    }
}


static void
evra_xtest_action_send_key (EvraXtestAction *action, gchar *keysymname)
{
  KeySym keysym;

  g_return_if_fail (keysymname != NULL);

  if (xdisplay == NULL)
    return;

  keysym = XStringToKeysym (keysymname);

  if (keysym == NoSymbol)
    {
      g_printerr ("Unknown Keysym \"%s\".\n", keysymname);
      return;
    }

  XTestGrabControl (xdisplay, True);
  XTestFakeKeyEvent (xdisplay, XKeysymToKeycode (xdisplay, keysym),
                     True, CurrentTime);
  XFlush (xdisplay);
  XTestFakeKeyEvent (xdisplay, XKeysymToKeycode (xdisplay, keysym),
                     False, CurrentTime);
  XTestGrabControl (xdisplay, False);
  XFlush (xdisplay);
}


